@@ -1,7 +1,7 @@
 /*
  * Asterisk -- An open source telephony toolkit.
  *
- * Copyright (C) 2007, Digium, Inc.
+ * Copyright (C) 2007 - 2008, Digium, Inc.
  *
  * Russell Bryant <russell@digium.com>
  *
@@ -63,7 +63,7 @@ extern "C" {
 
 #include "asterisk/event_defs.h"
 
-/*! 
+/*!
  * \brief Subscriber event callback type
  *
  * \param event the event being passed to the subscriber
@@ -73,11 +73,12 @@ extern "C" {
  */
 typedef void (*ast_event_cb_t)(const struct ast_event *event, void *userdata);
 
-/*! 
+/*!
  * \brief Subscribe to events
  *
  * \param event_type The type of events to subscribe to
  * \param cb The function to be called with events
+ * \param description Description of the subscription.
  * \param userdata data to be passed to the event callback
  *
  * The rest of the arguments to this function specify additional parameters for
@@ -109,19 +110,149 @@ typedef void (*ast_event_cb_t)(const struct ast_event *event, void *userdata);
  * information element, AST_EVENT_IE_MAILBOX, with the same string value
  * contained in peer->mailbox.  Also, the event callback will be passed a
  * pointer to the peer.
+ *
+ * \note A NULL description will cause this function to crash, so watch out!
  */
-struct ast_event_sub *ast_event_subscribe(enum ast_event_type event_type, 
-	ast_event_cb_t cb, void *userdata, ...);
+struct ast_event_sub *ast_event_subscribe(enum ast_event_type event_type,
+       ast_event_cb_t cb, char *description, void *userdata, ...);
+
+/*!
+ * \brief Allocate a subscription, but do not activate it
+ *
+ * \param type the event type to subscribe to
+ * \param cb the function to call when an event matches this subscription
+ * \param userdata data to pass to the provided callback
+ *
+ * This function should be used when you want to dynamically build a
+ * subscription.
+ *
+ * \return the allocated subscription, or NULL on failure
+ * \since 1.6.1
+ */
+struct ast_event_sub *ast_event_subscribe_new(enum ast_event_type type,
+	ast_event_cb_t cb, void *userdata);
+
+/*!
+ * \brief Destroy an allocated subscription
+ *
+ * \param sub the subscription to destroy
+ *
+ * This function should be used when a subscription is allocated with
+ * ast_event_subscribe_new(), but for some reason, you want to destroy it
+ * instead of activating it.  This could be because of an error when
+ * reading in the configuration for the dynamically built subscription.
+ * \since 1.6.1
+ */
+void ast_event_sub_destroy(struct ast_event_sub *sub);
+
+/*!
+ * \brief Append a uint parameter to a subscription
+ *
+ * \param sub the dynamic subscription allocated with ast_event_subscribe_new()
+ * \param ie_type the information element type for the parameter
+ * \param uint the value that must be present in the event to match this subscription
+ *
+ * \retval 0 success
+ * \retval non-zero failure
+ * \since 1.6.1
+ */
+int ast_event_sub_append_ie_uint(struct ast_event_sub *sub,
+	enum ast_event_ie_type ie_type, uint32_t uint);
+
+/*!
+ * \brief Append a bitflags parameter to a subscription
+ *
+ * \param sub the dynamic subscription allocated with ast_event_subscribe_new()
+ * \param ie_type the information element type for the parameter
+ * \param flags the flags that must be present in the event to match this subscription
+ *
+ * \retval 0 success
+ * \retval non-zero failure
+ * \since 1.8
+ */
+int ast_event_sub_append_ie_bitflags(struct ast_event_sub *sub,
+	enum ast_event_ie_type ie_type, uint32_t flags);
+
+/*!
+ * \brief Append a string parameter to a subscription
+ *
+ * \param sub the dynamic subscription allocated with ast_event_subscribe_new()
+ * \param ie_type the information element type for the parameter
+ * \param str the string that must be present in the event to match this subscription
+ *
+ * \retval 0 success
+ * \retval non-zero failure
+ * \since 1.6.1
+ */
+int ast_event_sub_append_ie_str(struct ast_event_sub *sub,
+	enum ast_event_ie_type ie_type, const char *str);
+
+/*!
+ * \brief Append a raw parameter to a subscription
+ *
+ * \param sub the dynamic subscription allocated with ast_event_subscribe_new()
+ * \param ie_type the information element type for the parameter
+ * \param data the data that must be present in the event to match this subscription
+ * \param raw_datalen length of data
+ *
+ * \retval 0 success
+ * \retval non-zero failure
+ * \since 1.6.1
+ */
+int ast_event_sub_append_ie_raw(struct ast_event_sub *sub,
+	enum ast_event_ie_type ie_type, void *data, size_t raw_datalen);
+
+/*!
+ * \brief Append an 'exists' parameter to a subscription
+ *
+ * \param sub the dynamic subscription allocated with ast_event_subscribe_new()
+ * \param ie_type the information element type that must be present in the event
+ *      for it to match this subscription.
+ *
+ * \retval 0 success
+ * \retval non-zero failure
+ * \since 1.6.1
+ */
+int ast_event_sub_append_ie_exists(struct ast_event_sub *sub,
+	enum ast_event_ie_type ie_type);
+
+/*!
+ * \brief Activate a dynamically built subscription
+ *
+ * \param sub the subscription to activate that was allocated using
+ *      ast_event_subscribe_new()
+ *
+ * Once a dynamically built subscription has had all of the parameters added
+ * to it, it should be activated using this function.
+ *
+ * \retval 0 success
+ * \retval non-zero failure
+ * \since 1.6.1
+ */
+int ast_event_sub_activate(struct ast_event_sub *sub);
 
 /*!
  * \brief Un-subscribe from events
  *
  * \param event_sub This is the reference to the subscription returned by
  *        ast_event_subscribe.
- * 
- * \return Nothing
+ *
+ * This function will remove a subscription and free the associated data
+ * structures.
+ *
+ * \return NULL for convenience.
+ * \version 1.6.1 return changed to NULL
  */
-void ast_event_unsubscribe(struct ast_event_sub *event_sub);
+struct ast_event_sub *ast_event_unsubscribe(struct ast_event_sub *event_sub);
+
+/*!
+ * \brief Get description for a subscription
+ *
+ * \param sub subscription
+ *
+ * \return string description of the subscription
+ */
+const char *ast_event_subscriber_get_description(struct ast_event_sub *sub);
 
 /*!
  * \brief Check if subscribers exist
@@ -156,7 +287,7 @@ void ast_event_unsubscribe(struct ast_event_sub *event_sub);
  * }
  * \endcode
  *
- * This example will check if there are any subscribers to MWI events for the 
+ * This example will check if there are any subscribers to MWI events for the
  * mailbox defined in the "mailbox" variable.
  */
 enum ast_event_subscriber_res ast_event_check_subscriber(enum ast_event_type event_type, ...);
@@ -178,6 +309,12 @@ enum ast_event_subscriber_res ast_event_check_subscriber(enum ast_event_type eve
  */
 void ast_event_report_subs(const struct ast_event_sub *sub);
 
+/*!
+ * \brief Dump the event cache for the subscriber
+ * \since 1.6.1
+ */
+void ast_event_dump_cache(const struct ast_event_sub *event_sub);
+
 /*!
  * \brief Create a new event
  *
@@ -195,6 +332,9 @@ void ast_event_report_subs(const struct ast_event_sub *sub);
  * because it makes no sense to do so.  So, a payload must also be specified
  * after the IE payload type.
  *
+ * \note The EID IE will be appended automatically when this function is used
+ *       with at least one IE specified.
+ *
  * \return This returns the event that has been created.  If there is an error
  *         creating the event, NULL will be returned.
  *
@@ -234,12 +374,13 @@ void ast_event_destroy(struct ast_event *event);
  * \param event the event to be queued
  *
  * \retval zero success
- * \retval non-zero failure
+ * \retval non-zero failure.  Note that the caller of this function is
+ *         responsible for destroying the event in the case of a failure.
  *
  * This function queues an event to be dispatched to all of the appropriate
  * subscribers.  This function will not block while the event is being
- * dispatched because a pool of event dispatching threads handle the event 
- * queue.
+ * dispatched because the event is queued up for a dispatching thread 
+ * to handle.
  */
 int ast_event_queue(struct ast_event *event);
 
@@ -248,42 +389,18 @@ int ast_event_queue(struct ast_event *event);
  *
  * \param event the event to be queued and cached
  *
- * The rest of the arguments to this function specify information elements to
- * use for determining which events in the cache that this event should replace.
- * All events in the cache that match the specified criteria will be removed from
- * the cache and then this one will be added.  The arguments are specified in 
- * the form:
- *
- * \code
- *    <enum ast_event_ie_type>, [enum ast_event_ie_pltype]
- * \endcode
- * and must end with AST_EVENT_IE_END.
- *
- * If the ie_type specified is *not* AST_EVENT_IE_END, then it must be followed
- * by a valid IE payload type.  If the payload type given is EXISTS, then all
- * events that contain that information element will be removed from the cache.
- * Otherwise, all events in the cache that contain an information element with
- * the same value as the new event will be removed.
- *
- * \note If more than one IE parameter is specified, they *all* must match for
- *       the event to be removed from the cache.
- *
- * Example usage:
- *
- * \code
- * ast_event_queue_and_cache(event,
- *     AST_EVENT_IE_MAILBOX, AST_EVENT_IE_PLTYPE_STR,
- *     AST_EVENT_IE_END);
- * \endcode
- *
- * This example queues and caches an event.  Any events in the cache that have
- * the same MAILBOX information element as this event will be removed.
- *
+ * \details
  * The purpose of caching events is so that the core can retain the last known
  * information for events that represent some sort of state.  That way, when
  * code needs to find out the current state, it can query the cache.
+ *
+ * The event API already knows which events can be cached and how to cache them.
+ *
+ * \retval 0 success
+ * \retval non-zero failure.  If failure is returned, the event must be destroyed
+ *         by the caller of this function.
  */
-int ast_event_queue_and_cache(struct ast_event *event, ...);
+int ast_event_queue_and_cache(struct ast_event *event);
 
 /*!
  * \brief Retrieve an event from the cache
@@ -359,6 +476,24 @@ int ast_event_append_ie_str(struct ast_event **event, enum ast_event_ie_type ie_
 int ast_event_append_ie_uint(struct ast_event **event, enum ast_event_ie_type ie_type,
 	uint32_t data);
 
+/*!
+ * \brief Append an information element that has a bitflags payload
+ *
+ * \param event the event that the IE will be appended to
+ * \param ie_type the type of IE to append
+ * \param bitflags the flags that are the payload of the IE
+ *
+ * \retval 0 success
+ * \retval -1 failure
+ * \since 1.8
+ *
+ * The pointer to the event will get updated with the new location for the event
+ * that now contains the appended information element.  If the re-allocation of
+ * the memory for this event fails, it will be set to NULL.
+ */
+int ast_event_append_ie_bitflags(struct ast_event **event, enum ast_event_ie_type ie_type,
+	uint32_t bitflags);
+
 /*!
  * \brief Append an information element that has a raw payload
  *
@@ -377,40 +512,108 @@ int ast_event_append_ie_uint(struct ast_event **event, enum ast_event_ie_type ie
 int ast_event_append_ie_raw(struct ast_event **event, enum ast_event_ie_type ie_type,
 	const void *data, size_t data_len);
 
+/*!
+ * \brief Append the global EID IE
+ *
+ * \param event the event to append IE to
+ *
+ * \note For ast_event_new() that includes IEs, this is done automatically
+ *       for you.
+ *
+ * \retval 0 success
+ * \retval -1 failure
+ */
+int ast_event_append_eid(struct ast_event **event);
+
 /*!
  * \brief Get the value of an information element that has an integer payload
  *
  * \param event The event to get the IE from
  * \param ie_type the type of information element to retrieve
- * 
+ *
  * \return This returns the payload of the information element with the given type.
  *         However, an IE with a payload of 0, and the case where no IE is found
  *         yield the same return value.
  */
 uint32_t ast_event_get_ie_uint(const struct ast_event *event, enum ast_event_ie_type ie_type);
 
+/*!
+ * \brief Get the value of an information element that has a bitflags payload
+ *
+ * \param event The event to get the IE from
+ * \param ie_type the type of information element to retrieve
+ *
+ * \return This returns the payload of the information element with the given type.
+ *         However, an IE with a payload of 0, and the case where no IE is found
+ *         yield the same return value.
+ */
+uint32_t ast_event_get_ie_bitflags(const struct ast_event *event, enum ast_event_ie_type ie_type);
+
 /*!
  * \brief Get the value of an information element that has a string payload
  *
  * \param event The event to get the IE from
  * \param ie_type the type of information element to retrieve
- * 
+ *
  * \return This returns the payload of the information element with the given type.
  *         If the information element isn't found, NULL will be returned.
  */
 const char *ast_event_get_ie_str(const struct ast_event *event, enum ast_event_ie_type ie_type);
 
+/*!
+ * \brief Get the hash for the string payload of an IE
+ *
+ * \param event The event to get the IE from
+ * \param ie_type the type of information element to retrieve the hash for
+ *
+ * \return This function returns the hash value as calculated by ast_str_hash()
+ *         for the string payload.  This is stored in the event to avoid
+ *         unnecessary string comparisons.
+ */
+uint32_t ast_event_get_ie_str_hash(const struct ast_event *event, enum ast_event_ie_type ie_type);
+
 /*!
  * \brief Get the value of an information element that has a raw payload
  *
  * \param event The event to get the IE from
  * \param ie_type the type of information element to retrieve
- * 
+ *
  * \return This returns the payload of the information element with the given type.
  *         If the information element isn't found, NULL will be returned.
  */
 const void *ast_event_get_ie_raw(const struct ast_event *event, enum ast_event_ie_type ie_type);
 
+/*!
+ * \brief Get the length of the raw payload for a particular IE
+ *
+ * \param event The event to get the IE payload length from
+ * \param ie_type the type of information element to get the length of
+ *
+ * \return If an IE of type ie_type is found, its payload length is returned.
+ *         Otherwise, 0 is returned.
+ */
+uint16_t ast_event_get_ie_raw_payload_len(const struct ast_event *event, enum ast_event_ie_type ie_type);
+
+/*!
+ * \brief Get the string representation of an information element type
+ *
+ * \param ie_type the information element type to get the string representation of
+ *
+ * \return the string representation of the information element type
+ * \since 1.6.1
+ */
+const char *ast_event_get_ie_type_name(enum ast_event_ie_type ie_type);
+
+/*!
+ * \brief Get the payload type for a given information element type
+ *
+ * \param ie_type the information element type to get the payload type of
+ *
+ * \return the payload type for the provided IE type
+ * \since 1.6.1
+ */
+enum ast_event_ie_pltype ast_event_get_ie_pltype(enum ast_event_ie_type ie_type);
+
 /*!
  * \brief Get the type for an event
  *
@@ -421,15 +624,60 @@ const void *ast_event_get_ie_raw(const struct ast_event *event, enum ast_event_i
  */
 enum ast_event_type ast_event_get_type(const struct ast_event *event);
 
+/*!
+ * \brief Get the string representation of the type of the given event
+ *
+ * \arg event the event to get the type of
+ *
+ * \return the string representation of the event type of the provided event
+ * \since 1.6.1
+ */
+const char *ast_event_get_type_name(const struct ast_event *event);
+
+/*!
+ * \brief Convert a string into an event type
+ *
+ * \param str the string to convert
+ * \param event_type an output parameter for the event type
+ *
+ * \retval 0 success
+ * \retval non-zero failure
+ * \since 1.6.1
+ */
+int ast_event_str_to_event_type(const char *str, enum ast_event_type *event_type);
+
+/*!
+ * \brief Convert a string to an IE type
+ *
+ * \param str the string to convert
+ * \param ie_type an output parameter for the IE type
+ *
+ * \retval 0 success
+ * \retval non-zero failure
+ * \since 1.6.1
+ */
+int ast_event_str_to_ie_type(const char *str, enum ast_event_ie_type *ie_type);
+
+/*!
+ * \brief Get the size of an event
+ *
+ * \param event the event to get the size of
+ *
+ * \return the number of bytes contained in the event
+ * \since 1.6.1
+ */
+size_t ast_event_get_size(const struct ast_event *event);
+
 /*!
  * \brief Initialize an event iterator instance
  *
  * \param iterator The iterator instance to initialize
  * \param event The event that will be iterated through
  *
- * \return Nothing
+ * \retval 0 Success, there are IEs available to iterate
+ * \retval -1 Failure, there are no IEs in the event to iterate
  */
-void ast_event_iterator_init(struct ast_event_iterator *iterator, const struct ast_event *event);
+int ast_event_iterator_init(struct ast_event_iterator *iterator, const struct ast_event *event);
 
 /*!
  * \brief Move iterator instance to next IE
@@ -452,7 +700,7 @@ int ast_event_iterator_next(struct ast_event_iterator *iterator);
 enum ast_event_ie_type ast_event_iterator_get_ie_type(struct ast_event_iterator *iterator);
 
 /*!
- * \brief Get the value of the current IE in the ierator as an integer payload
+ * \brief Get the value of the current IE in the iterator as an integer payload
  *
  * \param iterator The iterator instance
  *
@@ -460,6 +708,15 @@ enum ast_event_ie_type ast_event_iterator_get_ie_type(struct ast_event_iterator
  */
 uint32_t ast_event_iterator_get_ie_uint(struct ast_event_iterator *iterator);
 
+/*!
+ * \brief Get the value of the current IE in the iterator as a bitflags payload
+ *
+ * \param iterator The iterator instance
+ *
+ * \return This returns the payload of the information element as bitflags.
+ */
+uint32_t ast_event_iterator_get_ie_bitflags(struct ast_event_iterator *iterator);
+
 /*!
  * \brief Get the value of the current IE in the iterator as a string payload
  *
@@ -478,6 +735,15 @@ const char *ast_event_iterator_get_ie_str(struct ast_event_iterator *iterator);
  */
 void *ast_event_iterator_get_ie_raw(struct ast_event_iterator *iterator);
 
+/*!
+ * \brief Get the length of the raw payload for the current IE for an iterator
+ *
+ * \param iterator The IE iterator
+ *
+ * \return The payload length of the current IE
+ */
+uint16_t ast_event_iterator_get_ie_raw_payload_len(struct ast_event_iterator *iterator);
+
 #if defined(__cplusplus) || defined(c_plusplus)
 }
 #endif
