@@ -31,61 +31,65 @@ extern "C" {
 
 #include <sys/time.h>
 
+#include "asterisk/frame_defs.h"
 #include "asterisk/endian.h"
 #include "asterisk/linkedlists.h"
 
 struct ast_codec_pref {
-	char order[32];
-	char framing[32];
+	char order[sizeof(format_t) * 8];
+	char framing[sizeof(format_t) * 8];
 };
 
-/*! \page Def_Frame AST Multimedia and signalling frames
-	\section Def_AstFrame What is an ast_frame ?
- 	A frame of data read used to communicate between 
- 	between channels and applications.
-	Frames are divided into frame types and subclasses.
-
-	\par Frame types 
-	\arg \b VOICE:	Voice data, subclass is codec (AST_FORMAT_*)
-	\arg \b VIDEO:	Video data, subclass is codec (AST_FORMAT_*)
-	\arg \b DTMF:	A DTMF digit, subclass is the digit
-	\arg \b IMAGE:	Image transport, mostly used in IAX
-	\arg \b TEXT:	Text messages and character by character (real time text)
-	\arg \b HTML:	URL's and web pages
-	\arg \b MODEM:	Modulated data encodings, such as T.38 and V.150
-	\arg \b IAX:	Private frame type for the IAX protocol
-	\arg \b CNG:	Comfort noice frames
-	\arg \b CONTROL:	A control frame, subclass defined as AST_CONTROL_
-	\arg \b NULL:	Empty, useless frame
-
-	\par Files
-	\arg frame.h	Definitions
-	\arg frame.c	Function library
-	\arg \ref Def_Channel Asterisk channels
-	\section Def_ControlFrame Control Frames
-	Control frames send signalling information between channels
-	and devices. They are prefixed with AST_CONTROL_, like AST_CONTROL_FRAME_HANGUP
-	\arg \b HANGUP	The other end has hungup
-	\arg \b RING	Local ring
-	\arg \b RINGING	The other end is ringing
-	\arg \b ANSWER	The other end has answered
-	\arg \b BUSY	Remote end is busy
-	\arg \b TAKEOFFHOOK	Make it go off hook (what's "it" ? )
-	\arg \b OFFHOOK	Line is off hook
-	\arg \b CONGESTION	Congestion (circuit is busy, not available)
-	\arg \b FLASH	Other end sends flash hook
-	\arg \b WINK	Other end sends wink
-	\arg \b OPTION	Send low-level option
-	\arg \b RADIO_KEY	Key radio (see app_rpt.c)
-	\arg \b RADIO_UNKEY	Un-key radio (see app_rpt.c)
-	\arg \b PROGRESS	Other end indicates call progress
-	\arg \b PROCEEDING	Indicates proceeding
-	\arg \b HOLD	Call is placed on hold
-	\arg \b UNHOLD	Call is back from hold
-	\arg \b VIDUPDATE	Video update requested
-	\arg \b SRCUPDATE       The source of media has changed
-
-*/
+/*!
+ * \page Def_Frame AST Multimedia and signalling frames
+ * \section Def_AstFrame What is an ast_frame ?
+ * A frame of data read used to communicate between 
+ * between channels and applications.
+ * Frames are divided into frame types and subclasses.
+ *
+ * \par Frame types 
+ * \arg \b VOICE:  Voice data, subclass is codec (AST_FORMAT_*)
+ * \arg \b VIDEO:  Video data, subclass is codec (AST_FORMAT_*)
+ * \arg \b DTMF:   A DTMF digit, subclass is the digit
+ * \arg \b IMAGE:  Image transport, mostly used in IAX
+ * \arg \b TEXT:   Text messages and character by character (real time text)
+ * \arg \b HTML:   URL's and web pages
+ * \arg \b MODEM:  Modulated data encodings, such as T.38 and V.150
+ * \arg \b IAX:    Private frame type for the IAX protocol
+ * \arg \b CNG:    Comfort noice frames
+ * \arg \b CONTROL:A control frame, subclass defined as AST_CONTROL_
+ * \arg \b NULL:   Empty, useless frame
+ *
+ * \par Files
+ * \arg frame.h    Definitions
+ * \arg frame.c    Function library
+ * \arg \ref Def_Channel Asterisk channels
+ * \section Def_ControlFrame Control Frames
+ * Control frames send signalling information between channels
+ * and devices. They are prefixed with AST_CONTROL_, like AST_CONTROL_FRAME_HANGUP
+ * \arg \b HANGUP          The other end has hungup
+ * \arg \b RING            Local ring
+ * \arg \b RINGING         The other end is ringing
+ * \arg \b ANSWER          The other end has answered
+ * \arg \b BUSY            Remote end is busy
+ * \arg \b TAKEOFFHOOK     Make it go off hook (what's "it" ? )
+ * \arg \b OFFHOOK         Line is off hook
+ * \arg \b CONGESTION      Congestion (circuit is busy, not available)
+ * \arg \b FLASH           Other end sends flash hook
+ * \arg \b WINK            Other end sends wink
+ * \arg \b OPTION          Send low-level option
+ * \arg \b RADIO_KEY       Key radio (see app_rpt.c)
+ * \arg \b RADIO_UNKEY     Un-key radio (see app_rpt.c)
+ * \arg \b PROGRESS        Other end indicates call progress
+ * \arg \b PROCEEDING      Indicates proceeding
+ * \arg \b HOLD            Call is placed on hold
+ * \arg \b UNHOLD          Call is back from hold
+ * \arg \b VIDUPDATE       Video update requested
+ * \arg \b SRCUPDATE       The source of media has changed (RTP marker bit must change)
+ * \arg \b SRCCHANGE       Media source has changed (RTP marker bit and SSRC must change)
+ * \arg \b CONNECTED_LINE  Connected line has changed
+ * \arg \b REDIRECTING     Call redirecting information has changed.
+ */
 
 /*!
  * \brief Frame types 
@@ -128,13 +132,18 @@ enum {
 	AST_FRFLAG_HAS_TIMING_INFO = (1 << 0),
 };
 
+union ast_frame_subclass {
+	int integer;
+	format_t codec;
+};
+
 /*! \brief Data structure associated with a single frame of data
  */
 struct ast_frame {
 	/*! Kind of frame */
 	enum ast_frame_type frametype;				
 	/*! Subclass, frame dependent */
-	int subclass;				
+	union ast_frame_subclass subclass;
 	/*! Length of data */
 	int datalen;				
 	/*! Number of samples in this frame */
@@ -148,7 +157,7 @@ struct ast_frame {
 	/*! Optional source of frame for debugging */
 	const char *src;				
 	/*! Pointer to actual data */
-	void *data;		
+	union { void *ptr; uint32_t uint32; char pad[8]; } data;
 	/*! Global delivery time */		
 	struct timeval delivery;
 	/*! For placing in a linked list */
@@ -173,7 +182,7 @@ struct ast_frame {
  */
 #define	AST_FRAME_SET_BUFFER(fr, _base, _ofs, _datalen)	\
 	{					\
-	(fr)->data = (char *)_base + (_ofs);	\
+	(fr)->data.ptr = (char *)_base + (_ofs);	\
 	(fr)->offset = (_ofs);			\
 	(fr)->datalen = (_datalen);		\
 	}
@@ -230,55 +239,69 @@ extern struct ast_frame ast_null_frame;
 
 /* Data formats for capabilities and frames alike */
 /*! G.723.1 compression */
-#define AST_FORMAT_G723_1	(1 << 0)
+#define AST_FORMAT_G723_1     (1ULL << 0)
 /*! GSM compression */
-#define AST_FORMAT_GSM		(1 << 1)
+#define AST_FORMAT_GSM        (1ULL << 1)
 /*! Raw mu-law data (G.711) */
-#define AST_FORMAT_ULAW		(1 << 2)
+#define AST_FORMAT_ULAW       (1ULL << 2)
 /*! Raw A-law data (G.711) */
-#define AST_FORMAT_ALAW		(1 << 3)
+#define AST_FORMAT_ALAW       (1ULL << 3)
 /*! ADPCM (G.726, 32kbps, AAL2 codeword packing) */
-#define AST_FORMAT_G726_AAL2	(1 << 4)
+#define AST_FORMAT_G726_AAL2  (1ULL << 4)
 /*! ADPCM (IMA) */
-#define AST_FORMAT_ADPCM	(1 << 5)
+#define AST_FORMAT_ADPCM      (1ULL << 5)
 /*! Raw 16-bit Signed Linear (8000 Hz) PCM */
-#define AST_FORMAT_SLINEAR	(1 << 6)
+#define AST_FORMAT_SLINEAR    (1ULL << 6)
 /*! LPC10, 180 samples/frame */
-#define AST_FORMAT_LPC10	(1 << 7)
+#define AST_FORMAT_LPC10      (1ULL << 7)
 /*! G.729A audio */
-#define AST_FORMAT_G729A	(1 << 8)
+#define AST_FORMAT_G729A      (1ULL << 8)
 /*! SpeeX Free Compression */
-#define AST_FORMAT_SPEEX	(1 << 9)
+#define AST_FORMAT_SPEEX      (1ULL << 9)
 /*! iLBC Free Compression */
-#define AST_FORMAT_ILBC		(1 << 10)
+#define AST_FORMAT_ILBC       (1ULL << 10)
 /*! ADPCM (G.726, 32kbps, RFC3551 codeword packing) */
-#define AST_FORMAT_G726		(1 << 11)
+#define AST_FORMAT_G726       (1ULL << 11)
 /*! G.722 */
-#define AST_FORMAT_G722		(1 << 12)
-/*! Unsupported audio bits */
-#define AST_FORMAT_AUDIO_UNDEFINED	((1 << 13) | (1 << 14))
+#define AST_FORMAT_G722       (1ULL << 12)
+/*! G.722.1 (also known as Siren7, 32kbps assumed) */
+#define AST_FORMAT_SIREN7     (1ULL << 13)
+/*! G.722.1 Annex C (also known as Siren14, 48kbps assumed) */
+#define AST_FORMAT_SIREN14    (1ULL << 14)
 /*! Raw 16-bit Signed Linear (16000 Hz) PCM */
-#define AST_FORMAT_SLINEAR16	(1 << 15)
+#define AST_FORMAT_SLINEAR16  (1ULL << 15)
 /*! Maximum audio mask */
-#define AST_FORMAT_AUDIO_MASK   ((1 << 16)-1)
+#define AST_FORMAT_AUDIO_MASK 0xFFFF0000FFFFULL
 /*! JPEG Images */
-#define AST_FORMAT_JPEG		(1 << 16)
+#define AST_FORMAT_JPEG       (1ULL << 16)
 /*! PNG Images */
-#define AST_FORMAT_PNG		(1 << 17)
+#define AST_FORMAT_PNG        (1ULL << 17)
 /*! H.261 Video */
-#define AST_FORMAT_H261		(1 << 18)
+#define AST_FORMAT_H261       (1ULL << 18)
 /*! H.263 Video */
-#define AST_FORMAT_H263		(1 << 19)
+#define AST_FORMAT_H263       (1ULL << 19)
 /*! H.263+ Video */
-#define AST_FORMAT_H263_PLUS	(1 << 20)
+#define AST_FORMAT_H263_PLUS  (1ULL << 20)
 /*! H.264 Video */
-#define AST_FORMAT_H264		(1 << 21)
+#define AST_FORMAT_H264       (1ULL << 21)
 /*! MPEG4 Video */
-#define AST_FORMAT_MP4_VIDEO	(1 << 22)
-#define AST_FORMAT_VIDEO_MASK   (((1 << 25)-1) & ~(AST_FORMAT_AUDIO_MASK))
-/*! T.140 Text format - ITU T.140, RFC 4351*/
-#define AST_FORMAT_T140		(1 << 25)
-#define AST_FORMAT_TEXT_MASK   (((1 << 30)-1) & ~(AST_FORMAT_AUDIO_MASK) & ~(AST_FORMAT_VIDEO_MASK))
+#define AST_FORMAT_MP4_VIDEO  (1ULL << 22)
+#define AST_FORMAT_VIDEO_MASK ((((1ULL << 25)-1) & ~(AST_FORMAT_AUDIO_MASK)) | 0x7FFF000000000000ULL)
+/*! T.140 RED Text format RFC 4103 */
+#define AST_FORMAT_T140RED    (1ULL << 26)
+/*! T.140 Text format - ITU T.140, RFC 4103 */
+#define AST_FORMAT_T140       (1ULL << 27)
+/*! Maximum text mask */
+#define AST_FORMAT_MAX_TEXT   (1ULL << 28)
+#define AST_FORMAT_TEXT_MASK  (((1ULL << 30)-1) & ~(AST_FORMAT_AUDIO_MASK) & ~(AST_FORMAT_VIDEO_MASK))
+/*! G.719 (64 kbps assumed) */
+#define AST_FORMAT_G719	      (1ULL << 32)
+/*! SpeeX Wideband (16kHz) Free Compression */
+#define AST_FORMAT_SPEEX16    (1ULL << 33)
+/*! Raw mu-law data (G.711) */
+#define AST_FORMAT_TESTLAW    (1ULL << 47)
+/*! Reserved bit - do not use */
+#define AST_FORMAT_RESERVED   (1ULL << 63)
 
 enum ast_control_frame_type {
 	AST_CONTROL_HANGUP = 1,		/*!< Other end has hungup */
@@ -301,7 +324,32 @@ enum ast_control_frame_type {
 	AST_CONTROL_VIDUPDATE = 18,	/*!< Indicate video frame update */
 	_XXX_AST_CONTROL_T38 = 19,	/*!< T38 state change request/notification \deprecated This is no longer supported. Use AST_CONTROL_T38_PARAMETERS instead. */
 	AST_CONTROL_SRCUPDATE = 20,     /*!< Indicate source of media has changed */
-	AST_CONTROL_T38_PARAMETERS = 24, /*!< T38 state change request/notification with parameters */
+	AST_CONTROL_TRANSFER = 21,      /*!< Indicate status of a transfer request */
+	AST_CONTROL_CONNECTED_LINE = 22,/*!< Indicate connected line has changed */
+	AST_CONTROL_REDIRECTING = 23,    /*!< Indicate redirecting id has changed */
+	AST_CONTROL_T38_PARAMETERS = 24, /*! T38 state change request/notification with parameters */
+	AST_CONTROL_CC = 25, /*!< Indication that Call completion service is possible */
+	AST_CONTROL_SRCCHANGE = 26,  /*!< Media source has changed and requires a new RTP SSRC */
+	AST_CONTROL_READ_ACTION = 27, /*!< Tell ast_read to take a specific action */
+	AST_CONTROL_AOC = 28,           /*!< Advice of Charge with encoded generic AOC payload */
+	AST_CONTROL_END_OF_Q = 29,		/*!< Indicate that this position was the end of the channel queue for a softhangup. */
+};
+
+enum ast_frame_read_action {
+	AST_FRAME_READ_ACTION_CONNECTED_LINE_MACRO,
+};
+
+struct ast_control_read_action_payload {
+	/* An indicator to ast_read of what action to
+	 * take with the frame;
+	 */
+	enum ast_frame_read_action action;
+	/* The size of the frame's payload
+	 */
+	size_t payload_size;
+	/* A payload for the frame.
+	 */
+	unsigned char payload[0];
 };
 
 enum ast_control_t38 {
@@ -309,7 +357,8 @@ enum ast_control_t38 {
 	AST_T38_REQUEST_TERMINATE,	/*!< Terminate T38 on a channel (fax to voice) */
 	AST_T38_NEGOTIATED,		/*!< T38 negotiated (fax mode) */
 	AST_T38_TERMINATED,		/*!< T38 terminated (back to voice) */
-	AST_T38_REFUSED			/*!< T38 refused for some reason (usually rejected by remote end) */
+	AST_T38_REFUSED,		/*!< T38 refused for some reason (usually rejected by remote end) */
+	AST_T38_REQUEST_PARMS,		/*!< request far end T.38 parameters for a channel in 'negotiating' state */
 };
 
 enum ast_control_t38_rate {
@@ -337,6 +386,11 @@ struct ast_control_t38_parameters {
 	unsigned int transcoding_jbig:1;			/*!< Set if JBIG transcoding can be used */
 };
 
+enum ast_control_transfer {
+	AST_TRANSFER_SUCCESS = 0, /*!< Transfer request on the channel worked */
+	AST_TRANSFER_FAILED,      /*!< Transfer request on the channel failed */
+};
+
 #define AST_SMOOTHER_FLAG_G729		(1 << 0)
 #define AST_SMOOTHER_FLAG_BE		(1 << 1)
 
@@ -383,12 +437,44 @@ struct ast_control_t38_parameters {
 /*! Explicitly enable or disable echo cancelation for the given channel */
 #define	AST_OPTION_ECHOCAN		8
 
+/*! \brief Handle channel write data
+ * If a channel needs to process the data from a func_channel write operation
+ * after func_channel_write executes, it can define the setoption callback
+ * and process this option. A pointer to an ast_chan_write_info_t will be passed.
+ * */
+#define AST_OPTION_CHANNEL_WRITE 9
+
 /* !
  * Read-only. Allows query current status of T38 on the channel.
  * data: ast_t38state
  */
 #define AST_OPTION_T38_STATE		10
 
+/*! Request that the channel driver deliver frames in a specific format */
+#define AST_OPTION_FORMAT_READ          11
+
+/*! Request that the channel driver be prepared to accept frames in a specific format */
+#define AST_OPTION_FORMAT_WRITE         12
+
+/*! Request that the channel driver make two channels of the same tech type compatible if possible */
+#define AST_OPTION_MAKE_COMPATIBLE      13
+
+/*! Get or set the digit detection state of the channel */
+#define AST_OPTION_DIGIT_DETECT		14
+
+/*! Get or set the fax tone detection state of the channel */
+#define AST_OPTION_FAX_DETECT		15
+
+/*! Get the device name from the channel */
+#define AST_OPTION_DEVICE_NAME		16
+
+/*! Get the CC agent type from the channel */
+#define AST_OPTION_CC_AGENT_TYPE    17
+
+/*! Get or set the security options on a channel */
+#define AST_OPTION_SECURE_SIGNALING        18
+#define AST_OPTION_SECURE_MEDIA            19
+
 struct oprmode {
 	struct ast_channel *peer;
 	int mode;
@@ -413,7 +499,7 @@ struct ast_option_header {
 
 /*! \brief Definition of supported media formats (codecs) */
 struct ast_format_list {
-	int bits;	/*!< bitmask value */
+	format_t bits;	/*!< bitmask value */
 	char *name;	/*!< short name */
 	int samplespersecond; /*!< Number of samples per second (8000/16000) */
 	char *desc;	/*!< Description */
@@ -475,9 +561,9 @@ void ast_swapcopy_samples(void *dst, const void *src, int samples);
    little-endian and big-endian. */
 #if __BYTE_ORDER == __LITTLE_ENDIAN
 #define ast_frame_byteswap_le(fr) do { ; } while(0)
-#define ast_frame_byteswap_be(fr) do { struct ast_frame *__f = (fr); ast_swapcopy_samples(__f->data, __f->data, __f->samples); } while(0)
+#define ast_frame_byteswap_be(fr) do { struct ast_frame *__f = (fr); ast_swapcopy_samples(__f->data.ptr, __f->data.ptr, __f->samples); } while(0)
 #else
-#define ast_frame_byteswap_le(fr) do { struct ast_frame *__f = (fr); ast_swapcopy_samples(__f->data, __f->data, __f->samples); } while(0)
+#define ast_frame_byteswap_le(fr) do { struct ast_frame *__f = (fr); ast_swapcopy_samples(__f->data.ptr, __f->data.ptr, __f->samples); } while(0)
 #define ast_frame_byteswap_be(fr) do { ; } while(0)
 #endif
 
@@ -486,7 +572,7 @@ void ast_swapcopy_samples(void *dst, const void *src, int samples);
  * \param format id of format
  * \return A static string containing the name of the format or "unknown" if unknown.
  */
-char* ast_getformatname(int format);
+char* ast_getformatname(format_t format);
 
 /*! \brief Get the names of a set of formats
  * \param buf a buffer for the output string
@@ -496,21 +582,21 @@ char* ast_getformatname(int format);
  * ex: for format=AST_FORMAT_GSM|AST_FORMAT_SPEEX|AST_FORMAT_ILBC it will return "0x602 (GSM|SPEEX|ILBC)"
  * \return The return value is buf.
  */
-char* ast_getformatname_multiple(char *buf, size_t size, int format);
+char* ast_getformatname_multiple(char *buf, size_t size, format_t format);
 
 /*!
  * \brief Gets a format from a name.
  * \param name string of format
  * \return This returns the form of the format in binary on success, 0 on error.
  */
-int ast_getformatbyname(const char *name);
+format_t ast_getformatbyname(const char *name);
 
 /*! \brief Get a name from a format 
  * Gets a name from a format
  * \param codec codec number (1,2,4,8,16,etc.)
  * \return This returns a static string identifying the format on success, 0 on error.
  */
-char *ast_codec2str(int codec);
+char *ast_codec2str(format_t codec);
 
 /*! \name AST_Smoother 
 */
@@ -558,8 +644,8 @@ struct ast_frame *ast_smoother_read(struct ast_smoother *s);
 #endif
 /*@} Doxygen marker */
 
-struct ast_format_list *ast_get_format_list_index(int index);
-struct ast_format_list *ast_get_format_list(size_t *size);
+const struct ast_format_list *ast_get_format_list_index(int index);
+const struct ast_format_list *ast_get_format_list(size_t *size);
 void ast_frame_dump(const char *name, struct ast_frame *f, char *prefix);
 
 /*! \page AudioCodecPref Audio Codec Preferences
@@ -586,57 +672,65 @@ void ast_codec_pref_init(struct ast_codec_pref *pref);
  * \brief Codec located at a particular place in the preference index.
  * \arg \ref AudioCodecPref 
 */
-int ast_codec_pref_index(struct ast_codec_pref *pref, int index);
+format_t ast_codec_pref_index(struct ast_codec_pref *pref, int index);
 
 /*! \brief Remove audio a codec from a preference list */
-void ast_codec_pref_remove(struct ast_codec_pref *pref, int format);
+void ast_codec_pref_remove(struct ast_codec_pref *pref, format_t format);
 
 /*! \brief Append a audio codec to a preference list, removing it first if it was already there 
 */
-int ast_codec_pref_append(struct ast_codec_pref *pref, int format);
+int ast_codec_pref_append(struct ast_codec_pref *pref, format_t format);
 
 /*! \brief Prepend an audio codec to a preference list, removing it first if it was already there 
 */
-void ast_codec_pref_prepend(struct ast_codec_pref *pref, int format, int only_if_existing);
+void ast_codec_pref_prepend(struct ast_codec_pref *pref, format_t format, int only_if_existing);
 
 /*! \brief Select the best audio format according to preference list from supplied options. 
    If "find_best" is non-zero then if nothing is found, the "Best" format of 
    the format list is selected, otherwise 0 is returned. */
-int ast_codec_choose(struct ast_codec_pref *pref, int formats, int find_best);
+format_t ast_codec_choose(struct ast_codec_pref *pref, format_t formats, int find_best);
 
 /*! \brief Set packet size for codec
 */
-int ast_codec_pref_setsize(struct ast_codec_pref *pref, int format, int framems);
+int ast_codec_pref_setsize(struct ast_codec_pref *pref, format_t format, int framems);
 
 /*! \brief Get packet size for codec
 */
-struct ast_format_list ast_codec_pref_getsize(struct ast_codec_pref *pref, int format);
+struct ast_format_list ast_codec_pref_getsize(struct ast_codec_pref *pref, format_t format);
 
 /*! \brief Parse an "allow" or "deny" line in a channel or device configuration 
         and update the capabilities mask and pref if provided.
 	Video codecs are not added to codec preference lists, since we can not transcode
 	\return Returns number of errors encountered during parsing
  */
-int ast_parse_allow_disallow(struct ast_codec_pref *pref, int *mask, const char *list, int allowing);
+int ast_parse_allow_disallow(struct ast_codec_pref *pref, format_t *mask, const char *list, int allowing);
 
 /*! \brief Dump audio codec preference list into a string */
 int ast_codec_pref_string(struct ast_codec_pref *pref, char *buf, size_t size);
 
-/*! \brief Shift an audio codec preference list up or down 65 bytes so that it becomes an ASCII string */
+/*! \brief Shift an audio codec preference list up or down 65 bytes so that it becomes an ASCII string
+ * \note Due to a misunderstanding in how codec preferences are stored, this
+ * list starts at 'B', not 'A'.  For backwards compatibility reasons, this
+ * cannot change.
+ * \param pref A codec preference list structure
+ * \param buf A string denoting codec preference, appropriate for use in line transmission
+ * \param size Size of \a buf
+ * \param right Boolean:  if 0, convert from \a buf to \a pref; if 1, convert from \a pref to \a buf.
+ */
 void ast_codec_pref_convert(struct ast_codec_pref *pref, char *buf, size_t size, int right);
 
 /*! \brief Returns the number of samples contained in the frame */
 int ast_codec_get_samples(struct ast_frame *f);
 
 /*! \brief Returns the number of bytes for the number of samples of the given format */
-int ast_codec_get_len(int format, int samples);
+int ast_codec_get_len(format_t format, int samples);
 
 /*! \brief Appends a frame to the end of a list of frames, truncating the maximum length of the list */
 struct ast_frame *ast_frame_enqueue(struct ast_frame *head, struct ast_frame *f, int maxlen, int dupe);
 
 
 /*! \brief Gets duration in ms of interpolation frame for a format */
-static inline int ast_codec_interp_len(int format) 
+static inline int ast_codec_interp_len(format_t format) 
 { 
 	return (format == AST_FORMAT_ILBC) ? 30 : 20;
 }
@@ -663,14 +757,28 @@ int ast_frame_slinear_sum(struct ast_frame *f1, struct ast_frame *f2);
 /*!
  * \brief Get the sample rate for a given format.
  */
-static force_inline int ast_format_rate(int format)
+static force_inline int ast_format_rate(format_t format)
 {
-	if (format == AST_FORMAT_G722 || format == AST_FORMAT_SLINEAR16)
+	switch (format) {
+	case AST_FORMAT_G722:
+	case AST_FORMAT_SLINEAR16:
+	case AST_FORMAT_SIREN7:
+	case AST_FORMAT_SPEEX16:
 		return 16000;
-
-	return 8000;
+	case AST_FORMAT_SIREN14:
+		return 32000;
+	case AST_FORMAT_G719:
+		return 48000;
+	default:
+		return 8000;
+	}
 }
 
+/*!
+ * \brief Clear all audio samples from an ast_frame. The frame must be AST_FRAME_VOICE and AST_FORMAT_SLINEAR 
+ */
+int ast_frame_clear(struct ast_frame *frame);
+
 #if defined(__cplusplus) || defined(c_plusplus)
 }
 #endif
